#include "helpers/asciimapsource.hpp"
#include "asciimap_parser.hpp"
#include <fstream>
#include <iomanip>
#include <stdexcept>
#include <algorithm>

namespace dkh {
	namespace {
	} //unnamed namespace

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	AsciiMapSource::AsciiMapSource (const char* parFilename) {
		std::ifstream src(parFilename);
		parse_map_data(src);
		char* const base = reinterpret_cast<char*>(&m_wholedata.front());
		setg(base, base, base + m_wholedata.size());
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	AsciiMapSource::AsciiMapSource (const std::string& parFilename) {
		std::ifstream src(parFilename);
		parse_map_data(src);
		char* const base = reinterpret_cast<char*>(&m_wholedata.front());
		setg(base, base, base + m_wholedata.size());
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	AsciiMapSource::AsciiMapSource (std::istream& parData) {
		parse_map_data(parData);
		char* const base = reinterpret_cast<char*>(&m_wholedata.front());
		setg(base, base, base + m_wholedata.size());
	}

	///-------------------------------------------------------------------------
	///-------------------------------------------------------------------------
	void AsciiMapSource::parse_map_data (std::istream& parSrc) {
		typedef std::istream_iterator<char> DataIterator;
		typedef boost::spirit::multi_pass<DataIterator> WrappedIterator;

		parSrc >> std::noskipws;

		auto first = WrappedIterator(DataIterator(parSrc));
		auto last = WrappedIterator(DataIterator());
		implem::AsciiMapGrammar<WrappedIterator, MapTileType> grammar(first);
		const bool p = boost::spirit::qi::parse(
			first,
			last,
			grammar,
			m_wholedata
		);

		if (not p or last != first) {
			throw std::runtime_error("Invalid data: can't parse file");
		}

		//Only look at the front. Besides a bug in spirit makes the other counts
		//invalid, see
		//http://boost.2283326.n4.nabble.com/Possible-bug-in-line-pos-iterator-td4636592.html
		m_width = grammar.lengths.front();
		m_height = m_wholedata.size() / m_width;
	}
} //namespace dk
